# LICENSE HEADER MANAGED BY add-license-header
#
# SPDX-FileCopyrightText: Copyright 2024 German Cancer Research Center (DKFZ) and contributors.
# SPDX-License-Identifier: MIT
#

VERSION = (0, 6, 1)
__version__ = ".".join(map(str, VERSION))
__all__ = ["__version__"]
